#!/bin/bash

###########################################

INSTANCE="safesquid"
INSTANCE_DIR="/opt/safesquid/bin/safesquid"
CONFIG="/usr/local/safesquid/security/policies/config.xml"
LOGDIR="/var/log/safesquid"
RUN_DIR="/var/run/safesquid"
LOGLEVEL="268435455"
NATIVELOG="safesquid.log"
EXTENDEDLOG="extended.log"
PERFORMANCELOG="performance.log"
SAMPLING="1000"
SUPPORT_FILES="/var/log/messages"
#SAFESQUID_INI="/opt/safesquid/safesquid.ini"
SAFESQUID_INI="/opt/safesquid/default"
PIDFILE="/var/run/safesquid/safesquid.pid"
MODIFY_STARTUP="/opt/safesquid/startup.ini"
############################################


COMMAND="${PROG}"

SECTION=${INSTALL_DIR}/bin/section.xml

NATIVE_LOG_DIR=${LOGDIR}/native
NATIVE_LOG_FILE=${NATIVE_LOG_DIR}/${NATIVELOG}

EXTENDED_LOG_DIR=${LOGDIR}/extended
EXTENDED_LOG_FILE=${EXTENDED_LOG_DIR}/${EXTENDEDLOG}

PERFORMANCE_LOG_DIR=${LOGDIR}/performance
PERFORMANCE_LOG_FILE=${PERFORMANCE_LOG_DIR}/${PERFORMANCELOG}

REPORT_DIR=${LOGDIR}/stats
REPORT_FILE=${REPORT_DIR}/stats

PIDFILE=`echo -e "${RUN_DIR}/${INSTANCE}.pid"`

PROG=${INSTANCE_DIR}


TIME_STAMP ()
{
	echo -ne `date "+%Y-%m-%d::%H:%M:%S.%N"`"\t"
	echo "$*"
	
	return;
}


SUPPORT ()
{

		clear
		echo "Reading Parameters from startup params"	

		TIME_STAMP Please be patient, this can take a couple of minutes

		SUPPORT=/tmp/${INSTANCE}-`date +%Y-%m-%d-%H-%M-%S`
		mkdir -p ${SUPPORT}
#		STICKER &

		TIME_STAMP Creating folder: ${SUPPORT} to contain relevant files and information for support
				
		TIME_STAMP Copying startup.conf: ${SAFESQUID_INI} to ${SUPPORT}/
		cp -r ${SAFESQUID_INI} ${SUPPORT}/
			
		TIME_STAMP Copying modified startup.conf: ${MODIFY_STARTUP} to ${SUPPORT}/
		cp ${MODIFY_STARTUP} ${SUPPORT}/
	
		TIME_STAMP Copying CONFIG XML: ${CONFIG} to ${SUPPORT}/
		cp ${CONFIG} ${SUPPORT}/
		
		if [ -f  ${NATIVE_LOG_FILE} ]
		then
			TIME_STAMP Copying last ${SAMPLING} lines of ${NATIVE_LOG_FILE} to ${SUPPORT}/safesquid.log
			tail -n ${SAMPLING} ${NATIVE_LOG_FILE} > ${SUPPORT}/safesquid.log
		fi
				
		if [ -f ${EXTENDED_LOG_FILE} ]
		then
			TIME_STAMP Copying last ${SAMPLING} lines of ${EXTENDED_LOG_FILE} to ${SUPPORT}/extended.log
			tail -n ${SAMPLING} ${EXTENDED_LOG_FILE} > ${SUPPORT}/extended.log
		fi

		
		TIME_STAMP Copying last ${SAMPLING} lines of ${PERFORMANCE_LOG_FILE} to ${SUPPORT}/performance.log
		tail -n ${SAMPLING} ${PERFORMANCE_LOG_FILE} > ${SUPPORT}/performance.log
		
		[ -f ${SUPPORT}/temp.log ] && ${SUPPORT}/temp.log

		if [ -f ${NATIVE_LOG_FILE} ]
		then
			TIME_STAMP Parsing last ${SAMPLING} errors from ${NATIVE_LOG_FILE}
			tac ${NATIVE_LOG_FILE} | grep -m ${SAMPLING} -E --regexp="error:" >> ${SUPPORT}/temp.log			
		fi
		
		if [ -f ${NATIVE_LOG_FILE} ]
		then
			TIME_STAMP Parsing last ${SAMPLING} warnings from ${NATIVE_LOG_FILE}
			tac ${NATIVE_LOG_FILE} | grep -m ${SAMPLING} -E --regexp="warning:" >> ${SUPPORT}/temp.log
		
		fi

		if [ -f ${NATIVE_LOG_FILE} ]
		then
			TIME_STAMP Parsing last ${SAMPLING} advices from ${NATIVE_LOG_FILE}
			tac ${NATIVE_LOG_FILE} | grep -m ${SAMPLING} -E --regexp="advice:" >> ${SUPPORT}/temp.log			
		fi


		if [ -f ${SUPPORT}/temp.log ]
		then
			TIME_STAMP Merging \& Sorting ${SUPPORT}/error.log ${SUPPORT}/warning.log ${SUPPORT}/advice.log into ${SUPPORT}/diagnostics.log
			sort -n ${SUPPORT}/temp.log -o ${SUPPORT}/diagnostics.log
			TIME_STAMP Deleting temporary files: ${SUPPORT}/temp.log
			rm -f ${SUPPORT}/temp.log
		fi


		if [ -f ${PIDFILE}.nostart ]
		then
			TIME_STAMP Copying last 10000 lines from ${PIDFILE}.nostart into ${SUPPORT}/${INSTANCE}.pid.nostart
			tail -n 10000 ${PIDFILE}.nostart > ${SUPPORT}/${INSTANCE}.pid.nostart			
		fi

		TIME_STAMP Getting sysctl information into ${SUPPORT}/sysctl.txt
		sysctl -a > ${SUPPORT}/sysctl.txt 2> /dev/null
		> ${SUPPORT}/network.txt
		TIME_STAMP Getting the Information about Network Interfaces into ${SUPPORT}/network.txt
		netstat -v -e --interfaces >> ${SUPPORT}/network.txt
		TIME_STAMP Getting the Information about Network Routing Table into ${SUPPORT}/network.txt
		netstat -v -e --route >> ${SUPPORT}/network.txt
		TIME_STAMP Getting the Network Statistics into ${SUPPORT}/network.txt
		echo -e "\n--- Network Statistics ---\n" >> ${SUPPORT}/network.txt
		netstat -v -e --statistics >> ${SUPPORT}/network.txt
		TIME_STAMP Getting iptables NAT Configuration into ${SUPPORT}/network.txt
		echo -e "\n--- IPTABLES NAT rules ---\n" >> ${SUPPORT}/network.txt
		iptables -L -v -n -t nat >> ${SUPPORT}/network.txt
		TIME_STAMP Getting iptables MANGLE Configuration into ${SUPPORT}/network.txt
		echo -e "\n--- IPTABLES MANGLE rules ---\n" >> ${SUPPORT}/network.txt
		iptables -L -v -n -t mangle >> ${SUPPORT}/network.txt
		TIME_STAMP Getting iptables FILTER Configuration into ${SUPPORT}/network.txt
		echo -e "\n--- IPTABLES Filter rules ---\n" >> ${SUPPORT}/network.txt
		iptables -L -v -n -t filter >> ${SUPPORT}/network.txt
		> ${SUPPORT}/filesystem.txt
		TIME_STAMP Getting Disk Partitions Information into ${SUPPORT}/filesystem.txt
		echo -e "\n-- Disks and Partitions --\n" >> ${SUPPORT}/filesystem.txt
		fdisk -l >> ${SUPPORT}/filesystem.txt
		TIME_STAMP Getting Disk Usage Information into ${SUPPORT}/filesystem.txt
		echo -e "\n-- Disk Usage Information --\n" >> ${SUPPORT}/filesystem.txt
		df -Tha >> ${SUPPORT}/filesystem.txt
		TIME_STAMP Getting File System Mount Information into ${SUPPORT}/filesystem.txt
		echo -e "\n-- System Mount Information --\n" >> ${SUPPORT}/filesystem.txt
		mount -l >> ${SUPPORT}/filesystem.txt
		> ${SUPPORT}/systeminfo.txt
		TIME_STAMP Getting Linux Distribution Information into ${SUPPORT}/systeminfo.txt
		echo -e "\n-- Linux Distribution Information --\n" >> ${SUPPORT}/systeminfo.txt
		cat /etc/*-release >> ${SUPPORT}/systeminfo.txt
		TIME_STAMP Getting LIBC Information into ${SUPPORT}/systeminfo.txt
		echo -e "\n-- LIBC Information --\n" >> ${SUPPORT}/systeminfo.txt			
		$(ldd `which ls` | grep libc.so | awk '{print $3}') >> ${SUPPORT}/systeminfo.txt		
		TIME_STAMP Getting Kernel Information into ${SUPPORT}/systeminfo.txt
		echo -e "\n-- Kernel Information --\n" >> ${SUPPORT}/systeminfo.txt
		cat /proc/version >> ${SUPPORT}/systeminfo.txt
		TIME_STAMP Getting CPU information into ${SUPPORT}/systeminfo.txt
		echo -e "\n-- Processors Information --\n" >> ${SUPPORT}/systeminfo.txt
		cat /proc/cpuinfo >> ${SUPPORT}/systeminfo.txt
		TIME_STAMP Getting Memory Information into ${SUPPORT}/systeminfo.txt
		echo -e "\n-- Memory Information --\n" >> ${SUPPORT}/systeminfo.txt
		cat /proc/meminfo >> ${SUPPORT}/systeminfo.txt
		TIME_STAMP Getting System bootup information into ${SUPPORT}/systeminfo.txt
		echo -e "\n-- Bootup Information --\n" >> ${SUPPORT}/systeminfo.txt
		dmesg >> ${SUPPORT}/systeminfo.txt
		TIME_STAMP Getting SafeSquid Executable Info into ${SUPPORT}/safesquid.txt
		ls -l ${PROG} >> ${SUPPORT}/safesquid.txt
		echo -ne "SafeSquid Executable:\t" >> ${SUPPORT}/safesquid.txt
		TIME_STAMP Locating the SafeSquid Executable
		readlink -f ${PROG} >> ${SUPPORT}/safesquid.txt
		TIME_STAMP Checking the Version \& Build of the SafeSquid Executable
		echo -ne "Executable Version & Build:\t" >> ${SUPPORT}/safesquid.txt
		${PROG} -v >> ${SUPPORT}/safesquid.txt
		TIME_STAMP Getting the Library Dependencies of the SafeSquid Executable into ${SUPPORT}/safesquid.txt
		echo -ne "Library Dependencies:\n" >> ${SUPPORT}/safesquid.txt
		TIME_STAMP Getting the List of NEEDED libraries into ${SUPPORT}/safesquid.txt
		objdump -p ${PROG} | grep "NEEDED" >> ${SUPPORT}/safesquid.txt
		TIME_STAMP Getting the Information of SafeSquid\'s Dynamically Linked Libraries into ${SUPPORT}/safesquid.txt
		echo -ne "Library Resolution:\n" >> ${SUPPORT}/safesquid.txt
		ldd -v ${PROG} >> ${SUPPORT}/safesquid.txt

		for DEPENDENCY in `objdump -p ${PROG} | grep "NEEDED" | awk '{print $2}'`
		do 
			LIBS=`whereis -b ${DEPENDENCY} | awk '{print $2}'`
			echo -e "--- Library info for ${LIBS} ---" >> ${SUPPORT}/safesquid.txt; 
			objdump -gWpCltT --special-syms --dynamic-syms ${LIBS} 2> /dev/null >> ${SUPPORT}/safesquid.txt; 
		done
			
		wait
		[ "x${SUPPORT_FILES}" != "x" ] && GET_OTHER_FILES


		TIME_STAMP Creating the tar-ball for sending to SafeSquid Technical Support
		tar -czf ${SUPPORT}.tar.gz ${SUPPORT} 2> /dev/null
		TIME_STAMP Extracted contents of the support tar-ball may be confirmed from ${SUPPORT}
		wait
		
		cat <<- _EOF
		Please send ${SUPPORT}.tar.gz to SafeSquid Technical Support for trouble-shooting
		Extracted contents of the support tar-ball may be validated for privacy breach from ${SUPPORT}
		${SUPPORT} may be deleted whenever required.

		
		Thank You! for your patience.
		_EOF

	
	return;

}



MAIN(){
	
	SUPPORT
	
}

MAIN
